package com.alivc.rtc.linux;

public class AliRTCLinuxEngineImpl extends AliRTCLinuxEngine {

    interface InternalListener {

        void onError(int error_code);

        void onWarning(int warning_code);

        void onJoinChannelResult(int result);

        void onLeaveChannelResult(int result);

        void onRemoteUserOnLineNotify(String uid);

        void onRemoteUserOffLineNotify(String uid);

        void onPublishChangedNotify(int result, boolean isPublished);

        void onAudioFrameReceived(int type, long frame_ms, int channels, int sample_bits, int sample_rates, int samples, int size, byte[] buf);

        void onVideoFrameReceived(String uid, int track, int type, int frame_ms, int frame_num, int size, byte[] buf);
    }

    private InternalListener mInternalListener = new InternalListener() {
        @Override
        public void onError(int error_code) {
            if (mListener != null) {
                mListener.onError(error_code);
            }
        }

        @Override
        public void onWarning(int warning_code) {
            if (mListener != null) {
                mListener.onWarning(warning_code);
            }
        }

        @Override
        public void onJoinChannelResult(int result) {
            if (mListener != null) {
                mListener.onJoinChannelResult(result);
            }
        }

        @Override
        public void onLeaveChannelResult(int result) {
            if (mListener != null) {
                mListener.onLeaveChannelResult(result);
            }
        }

        @Override
        public void onRemoteUserOnLineNotify(String uid) {
            if (mListener != null) {
                mListener.onRemoteUserOnLineNotify(uid);
            }
        }

        @Override
        public void onRemoteUserOffLineNotify(String uid) {
            if (mListener != null) {
                mListener.onRemoteUserOffLineNotify(uid);
            }
        }

        @Override
        public void onPublishChangedNotify(int result, boolean isPublished) {
            if (mListener != null) {
                mListener.onPublishChangedNotify(result, isPublished);
            }
        }

        @Override
        public void onAudioFrameReceived(int type, long frameMs, int channels, int sampleBits, int sample_rates, int samples, int size, byte[] buf) {
            if (mListener != null) {
                AudioFrame frame = new AudioFrame();
                frame.type = AudioFrameType.getByValue(type);
                frame.pcm = new AudioPcmFrame(frameMs, sample_rates, samples);
                frame.pcm.channels = channels;
                frame.pcm.sample_bits = sampleBits;
                frame.pcm.pcmBuf = buf;
                frame.pcm.pcmBufSize = size;
                mListener.onAudioFrameReceived(frame);
            }
        }

        @Override
        public void onVideoFrameReceived(String uid, int track, int type, int frame_ms, int frame_num, int size, byte[] buf) {
            if (mListener != null) {
                VideoFrame frame = new VideoFrame();
                frame.type = VideoFrameType.getByValue(type);
                frame.track = RecordingVideoTrackType.getByValue(track);
                frame.h264 = new VideoH264Frame();
                frame.h264.frame_ms = frame_ms;
                frame.h264.frame_num = frame_num;
                frame.h264.buf = buf;
                frame.h264.bufSize = size;
                mListener.onVideoFrameReceived(uid, frame);
            }
        }
    };
    private AliRTCLinuxEngineListener mListener;
    private static final String TAG = AliRTCLinuxEngineImpl.class.getSimpleName();
    private long mNativeHandle;
    private AliRTCLinuxNative mJni = new AliRTCLinuxNative();

    AliRTCLinuxEngineImpl(AliRTCLinuxEngineListener listener, int lowPort, int highPort, String logPath, String coreServicePath, boolean h5mode) {
        mListener = listener;
        mNativeHandle = mJni.createAliRTCEngineInstance(mInternalListener, lowPort, highPort, logPath, coreServicePath, h5mode);
        if (mNativeHandle <= 0) {
            mNativeHandle = 0;
        }
    }

    @Override
    public void destroy() {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return;
        }
        mJni.destroy(mNativeHandle);
    }

    @Override
    public int joinChannel(AuthInfo authInfo, JoinChannelConfig config) {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return -1;
        }
        return mJni.joinChannel(mNativeHandle, authInfo, config.isAudioOnly, config.isVideoOnly,
                config.channelProfile.getValue(), config.clientRole.getValue(), config.publishMode.getValue(),
                config.recordingMode.getValue(), config.recordingVideoFormat.getValue(), config.recordingAudioFormat.getValue());
    }

    @Override
    public int leaveChannel() {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return -1;
        }
        return mJni.leaveChannel(mNativeHandle);
    }

    @Override
    public void setVideoProfile(VideoProfile profile, VideoTrack track) {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return;
        }
        mJni.setVideoProfile(mNativeHandle, profile.getValue(), track.getValue());
    }

    @Override
    public int getVideoProfile(VideoTrack track) {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return -1;
        }
        return mJni.getVideoProfile(mNativeHandle, track.getValue());
    }

    @Override
    public void configLocalCameraPublish(boolean enable) {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return;
        }
        mJni.configLocalCameraPublish(mNativeHandle, enable);
    }

    @Override
    public boolean isLocalCameraPublishEnabled() {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return false;
        }
        return mJni.isLocalCameraPublishEnabled(mNativeHandle);
    }

    @Override
    public void configLocalScreenPublish(boolean enable) {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return;
        }
        mJni.configLocalScreenPublish(mNativeHandle, enable);
    }

    @Override
    public boolean isLocalScreenPublishEnabled() {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return false;
        }
        return mJni.isLocalScreenPublishEnabled(mNativeHandle);
    }

    @Override
    public void configLocalAudioPublish(boolean enable) {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return;
        }
        mJni.configLocalAudioPublish(mNativeHandle, enable);
    }

    @Override
    public boolean isLocalAudioPublishEnabled() {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return false;
        }
        return mJni.isLocalAudioPublishEnabled(mNativeHandle);
    }

    @Override
    public int configLocalSimulcast(boolean enabled, VideoTrack track) {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return -1;
        }
        return mJni.configLocalSimulcast(mNativeHandle, enabled, track.getValue());
    }

    @Override
    public boolean isLocalSimulcastEnabled() {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return false;
        }
        return mJni.isLocalSimulcastEnabled(mNativeHandle);
    }

    @Override
    public int setExternalVideoSource(boolean enable, boolean useTexture, VideoSource source, RenderMode renderMode) {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return -1;
        }
        return mJni.setExternalVideoSource(mNativeHandle, enable, useTexture, source.getValue(), renderMode.getValue());
    }

    @Override
    public int pushExternalVideoFrame(VideoDataSample frame, VideoSource source) {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return -1;
        }
        return mJni.pushExternalVideoFrame(mNativeHandle, frame.format.getValue(), frame.bufferType.getValue(), frame.data, frame.dataLen, frame.strideY, frame.strideU, frame.strideV, frame.width, frame.height, frame.rotation, frame.timeStamp, source.getValue());
    }

    @Override
    public int setExternalAudioSource(boolean enable, int sampleRate, int channelsPerFrame) {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return -1;
        }
        return mJni.setExternalAudioSource(mNativeHandle, enable, sampleRate, channelsPerFrame);
    }

    @Override
    public int pushExternalAudioFrameRawData(byte[] audioSamples, int sampleLength, long timestamp) {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return -1;
        }
        return mJni.pushExternalAudioFrameRawData(mNativeHandle, audioSamples, sampleLength, timestamp);
    }

    @Override
    public int setExternalAudioPublishVolume(int volume) {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return -1;
        }
        return mJni.setExternalAudioPublishVolume(mNativeHandle, volume);
    }

    @Override
    public int getExternalAudioPublishVolume() {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return -1;
        }
        return mJni.getExternalAudioPublishVolume(mNativeHandle);
    }

    @Override
    public int publish() {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return -1;
        }
        return mJni.publish(mNativeHandle);
    }

    @Override
    public int startRecording() {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return -1;
        }
        return mJni.startRecording(mNativeHandle);
    }

    @Override
    public int stopRecording() {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return -1;
        }
        return mJni.stopRecording(mNativeHandle);
    }

    @Override
    public AliRTCLinuxMediaPlayer createMediaPlayer() {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return null;
        }
        return new AliRTCLinuxMediaPlayer(mJni.createMediaPlayer(mNativeHandle));
    }

    @Override
    public int destroyMediaPlayer(AliRTCLinuxMediaPlayer player) {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return -1;
        }
        if (player != null) {
            return mJni.destroyMediaPlayer(mNativeHandle, player.getNativeHandle());
        }
        System.out.println(TAG + " - player is null");
        return -1;
    }
}
