package com.alivc.rtc.linux;

public class AliRTCLinuxMediaPlayer {

    public static final int ERROR_INVALID_STATE = 0x01090001;
    public static final int ERROR_INVALID_PARAM = 0x01090002;
    public static final int ERROR_FILE_OPEN_FAILED = 0x01090003;

    public enum MediaPlayerState {
        MediaPlayerStateIdle(0),
        MediaPlayerStateInited(1),
        MediaPlayerStatePrepareing(2),
        MediaPlayerStatePrepared(3),
        MediaPlayerStateStarted(4),
        MediaPlayerStatePaused(5),
        MediaPlayerStateCompleted(6),
        MediaPlayerStateStopped(7),
        MediaPlayerStateError(8),
        MediaPlayerStateEnd(9);

        private int value;

        MediaPlayerState(int value) {
            this.value = value;
        }

        public int getValue() {
            return value;
        }
    }

    public enum MediaPlayerEvent {
        MediaPlayerEventBufferingStart(100),
        MediaPlayerEventBufferingEnd(200),
        MediaPlayerEventSeekComplete(300),
        MediaPlayerEventErrorOccured(900);
        private int value;

        MediaPlayerEvent(int value) {
            this.value = value;
        }

        public int getValue() {
            return value;
        }

        public static MediaPlayerEvent getByValue(int value) {
            for (MediaPlayerEvent e : values()) {
                if (e.value == value) {
                    return e;
                }
            }
            return null;
        }
    }

    interface InternalListener {

        void onStateChanged(int state);

        void onEventNotify(int event, int errorCode);

        void onPlayingProgress(long millisecond);

        void onVideoStreamChanged(int width, int height, int fps);
    }

    private InternalListener mInternalListener = new InternalListener() {
        @Override
        public void onStateChanged(int state) {
            if (mListener != null) {
                mListener.onStateChanged(AliRTCLinuxMediaPlayer.this, MediaPlayerState.values()[state]);
            }
        }

        @Override
        public void onEventNotify(int event, int errorCode) {
            if (mListener != null) {
                mListener.onEventNotify(AliRTCLinuxMediaPlayer.this, MediaPlayerEvent.getByValue(event), errorCode);
            }
        }

        @Override
        public void onPlayingProgress(long millisecond) {
            if (mListener != null) {
                mListener.onPlayingProgress(AliRTCLinuxMediaPlayer.this, millisecond);
            }
        }

        @Override
        public void onVideoStreamChanged(int width, int height, int fps) {
            if (mListener != null) {
                mListener.onVideoStreamChanged(AliRTCLinuxMediaPlayer.this, width, height, fps);
            }
        }

    };
    private AliRTCLinuxMediaPlayerListener mListener;
    private static final String TAG = AliRTCLinuxMediaPlayer.class.getSimpleName();
    private long mNativeHandle;
    private AliRTCLinuxNative mJni = new AliRTCLinuxNative();

    AliRTCLinuxMediaPlayer(long handle) {
        mNativeHandle = handle;
    }

    long getNativeHandle() {
        return mNativeHandle;
    }

    public int setEventHandler(AliRTCLinuxMediaPlayerListener eventHandler) {
        mListener = eventHandler;
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return -1;
        }
        return mJni.setEventHandler(mNativeHandle, mInternalListener);
    }

    public int loadResource(String path) {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return -1;
        }
        return mJni.loadResource(mNativeHandle, path);
    }

    public int start() {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return -1;
        }
        return mJni.start(mNativeHandle);
    }

    public int stop() {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return -1;
        }
        return mJni.stop(mNativeHandle);
    }

    public int pause() {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return -1;
        }
        return mJni.pause(mNativeHandle);
    }

    public int resume() {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return -1;
        }
        return mJni.resume(mNativeHandle);
    }

    public int seekTo(long millisecond) {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return -1;
        }
        return mJni.seekTo(mNativeHandle, millisecond);
    }

    public int setVideoSource(boolean enable, AliRTCLinuxEngine.VideoSource source, AliRTCLinuxEngine.RenderMode renderMode) {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return -1;
        }
        return mJni.setVideoSource(mNativeHandle, enable, source.getValue(), renderMode.getValue());
    }

    public int enableAudioSource(boolean enable) {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return -1;
        }
        return mJni.enableAudioSource(mNativeHandle, enable);
    }

    public int setVolume(int volume) {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return -1;
        }
        return mJni.setVolume(mNativeHandle, volume);
    }

    public long getDuration() {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return -1;
        }
        return mJni.getDuration(mNativeHandle);
    }

    public long getCurrentPlaybackTime() {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return -1;
        }
        return mJni.getCurrentPlaybackTime(mNativeHandle);
    }

    public int getCurrentVomume() {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return -1;
        }
        return mJni.getCurrentVomume(mNativeHandle);
    }

    public MediaPlayerState getCurrentPlaybackState() {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return MediaPlayerState.MediaPlayerStateIdle;
        }
        int state = mJni.getCurrentPlaybackState(mNativeHandle);
        return MediaPlayerState.values()[state];
    }

    public int getIndex() {
        if (mNativeHandle == 0) {
            System.out.println(TAG + " - native handle not initialize");
            return -1;
        }
        return mJni.getIndex(mNativeHandle);
    }
}
